#!/usr/bin/python3
# -*- coding: UTF-8 -*-
# Copyright (C) 2021  Avencall
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,

from builtins import object
from xivo.agi import AGI

import json
import sys
import traceback
import re
import dns.resolver
import dns.tsigkeyring
import dns.update
import dns.rdataset
import dns.rdtypes.IN.NAPTR
import dns.rdtypes.ANY.TXT
import dns.rdataclass
import dns.rdatatype
import dns.zone
import dns.query
TH_DIR = '/etc/thales'
TH_FULL_CONF = f'{TH_DIR}/thales-conf.json'

def init_config(configpath):
    result = {}
    with open(configpath, 'r') as configfile:
        result.update(json.load(configfile))
    return result

def dns_lookup(type, number_formatted, dns_section):
    """requests the dns asking whether it contains a line of the given type associated with the given number

    Args:
        type (str): 'TXT' or 'NAPTR'
        number_formatted (str): formatted number
        dns_section (str): the dns section we're looking into

    Returns:
        str: text output of the request
    """
    try:
        thales_resolver = dns.resolver.Resolver()
        line_with_domain = f"{number_formatted}.{dns_section}"
        answer = thales_resolver.resolve(line_with_domain, type)
        return answer.response.to_text()
    except dns.resolver.NoAnswer:
        return None
    except dns.resolver.NXDOMAIN:
        return None
    except Exception:
        exc_type, exc_value, exc_traceback = sys.exc_info()
        cause = repr(traceback.format_exception(exc_type, exc_value, exc_traceback))
        sys.exit(1)

def handle_location_call(agi, thales_conf, input_number, dns_section):
    NAPTR_OUTPUT = dns_lookup('NAPTR', input_number, dns_section)
    if NAPTR_OUTPUT is not None:
        agi.set_variable('TH_DNS_ANSWER',"Found")
        [number, domain] = extract_number_and_domain(NAPTR_OUTPUT)
        agi.set_variable('TH_DST_NUM_FROM_DNS',number)
        agi.set_variable('TH_DST_DOMAIN_FROM_DNS',domain)
    else:
        agi.set_variable('TH_DNS_ANSWER',"Not Found")

def extract_number_and_domain(dns_answer):
    answer_line = dns_answer[dns_answer.find('ANSWER'):dns_answer.find('AUTHORITY')]
    [number_part, domain_part] = answer_line.split('@')
    cutting_index =  len(number_part) - number_part[::-1].find(':') 
    number = number_part[cutting_index:]
    domain = domain_part[0:domain_part.find('!')]
    return [number, domain]

def handle_junction_call(agi, thales_conf, input_number):
    TXT_OUTPUT = dns_lookup('TXT', input_number,'jonctions.local')
    if TXT_OUTPUT is not None:
        agi.set_variable('TH_DNS_ANSWER',"Found")
        [prefix, ip] = extract_prefix_and_ip(TXT_OUTPUT)
        agi.set_variable('TH_JCT_PREFIX_FROM_DNS',prefix)
        agi.set_variable('TH_JCT_IP_FROM_DNS',ip)
    else:
        agi.set_variable('TH_DNS_ANSWER',"Not Found")

def extract_prefix_and_ip(dns_answer):
    [prefix_part, ip_part, trunkrate_part] = dns_answer.split(',')
    prefix = prefix_part[prefix_part.find(':')+1:]
    ip = ip_part[ip_part.find(':')+1:]
    return [prefix,ip]

def reverse_and_split_number(number):
    splitted = ""
    for digit_or_star in number[::-1]:
        splitted+=digit_or_star+'.'
    return splitted[:-1]

def main():
    # Init AGI
    agi = AGI()

    # Get config
    thales_conf = init_config(TH_FULL_CONF)

    try:
        if len(sys.argv) != 3 and len(sys.argv) != 4:
            sys.exit(1)
        if sys.argv[1] == 'findLocation':
            handle_location_call(agi, thales_conf, sys.argv[2], sys.argv[3])
        elif sys.argv[1] == 'findLocationOperator':
            handle_location_call(agi, thales_conf, reverse_and_split_number(sys.argv[2]), sys.argv[3])
        elif sys.argv[1] == 'getJunctionInfos':
            handle_junction_call(agi, thales_conf, sys.argv[2])
        else:
            sys.exit(1)
    except Exception:
        exc_type, exc_value, exc_traceback = sys.exc_info()
        cause = repr(traceback.format_exception(exc_type, exc_value, exc_traceback))
        sys.exit(1)
    sys.exit(0)

if __name__ == '__main__':
    main()
