#!/usr/bin/python3
# -*- coding: UTF-8 -*-
# Copyright (C) 2021  Avencall
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,

from builtins import object
from xivo.agi import AGI

import logging
import json
import sys
import traceback
import re
import dns.resolver
import dns.tsigkeyring
import dns.update
import dns.rdataset
import dns.rdtypes.IN.NAPTR
import dns.rdtypes.ANY.TXT
import dns.rdataclass
import dns.rdatatype
import dns.zone
import dns.query
import hashlib

DEBUG_MODE = False
LOGFILE = '/var/log/thales-user-subscription.log'
TH_DIR = '/etc/thales'
TH_FULL_CONF = f'{TH_DIR}/thales-conf.json'

logger = None
class Syslogger(object):

    def write(self, data):
        global logger
        logger.error(data)


def init_config(configpath):
    result = {}
    with open(configpath, 'r') as configfile:
        result.update(json.load(configfile))
    return result


def init_logging(debug_mode):
    if debug_mode:
        logger.setLevel(logging.DEBUG)
    else:
        logger.setLevel(logging.INFO)

    logfilehandler = logging.FileHandler(LOGFILE)

    formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    logfilehandler.setFormatter(formatter)
    logger.addHandler(logfilehandler)

    syslogger = Syslogger()
    sys.stderr = syslogger


def report_error(agi, cause, error_id = -1):
    """Throws a fatal error with details"""
    agi.verbose("Error: %s", cause)
    logger.error(cause)
    agi.set_variable('user-suscribe-result', f'{error_id}')
    agi.set_variable('error-cause', cause)
    sys.exit(1)


def format_subscriber_for_enum(number):
    """
    Returns the full Thales number in reverse order separated by dots.
    It is displayed this way in the DNS.
    """
    full_number = '2' + number
    return '.'.join(full_number[::-1])


def code_match(requestedCode, output):
    match = '"(.+?)"'
    code = re.search(match, output)
    if code:
        logger.debug(f"Code found : {code.group(1)}")
        return code.group(1) == f"CodePersonnel:{requestedCode}"
    else:
        logger.debug(f"No Code found for match {match},  Refuse update")
        return False

def dns_lookup(agi, config, thales_resolver, type, subscriber_enum_formatted):
    """requests the dns asking whether it contains a line of the given type associated with the given number

    Args:
        type (str): 'TXT' or 'NAPTR'
        subscriber_enum_formatted (str): formatted number

    Returns:
        str: text output of the request
    """
    logger.debug(f'DNS lookup for number {subscriber_enum_formatted} with type {type}')
    try:
        line_with_domain = f"{subscriber_enum_formatted}.{config['dns_zone']}"
        answer = thales_resolver.resolve(line_with_domain, type)
        return answer.response.to_text()
    except dns.resolver.NoAnswer:
        logger.debug(f'Type not found for this line in dns')
        return None
    except dns.resolver.NXDOMAIN:
        logger.debug(f'The line does not exist in dns')
        return None
    except Exception:
        exc_type, exc_value, exc_traceback = sys.exc_info()
        cause = repr(traceback.format_exception(exc_type, exc_value, exc_traceback))
        report_error(agi, cause)

def subscribing_user(config, update, subscriberNumber, NAPTR_output, TXT_output, subscriber_enum_formatted, personalCode):
    logger.info(f'Start subscribing user : {subscriberNumber}')

    dataset_naptr = dns.rdataset.Rdataset(dns.rdataclass.IN, dns.rdatatype.NAPTR, ttl=config['dns_ttl'])
    rdata_naptr = dns.rdtypes.IN.NAPTR.NAPTR(dns.rdataclass.IN, dns.rdatatype.NAPTR, 100, 10, "U", "E2U+sip",
                                             f"!^.*$!sip:2{subscriberNumber}@{config['station_ip']}!", dns.name.root)
    dataset_naptr.add(rdata_naptr, config['dns_ttl'])
    if NAPTR_output is None:
        update.add(f"{subscriber_enum_formatted}.{config['dns_zone']}.", dataset_naptr)
    else:
        update.replace(f"{subscriber_enum_formatted}.{config['dns_zone']}.", dataset_naptr)

    if TXT_output is None:
        dataset_txt = dns.rdataset.Rdataset(dns.rdataclass.IN, dns.rdatatype.TXT , ttl=config['dns_ttl'])
        rdata_txt = dns.rdtypes.ANY.TXT.TXT(dns.rdataclass.IN, dns.rdatatype.TXT , f"CodePersonnel:{personalCode}")
        dataset_txt.add(rdata_txt, config['dns_ttl'])
        update.add(f"{subscriber_enum_formatted}.{config['dns_zone']}.", dataset_txt)

    return update


def unsubscribing_user(config, update, subscriberNumber, NAPTR_output, subscriber_enum_formatted):
    logger.info(f'Start unsubscribing user : {subscriberNumber}')
    update.delete(f"{subscriber_enum_formatted}.{config['dns_zone']}.", 'NAPTR')
    return update


def main():
    # Init AGI
    agi = AGI()
    agi.verbose("before logger init")

    # Instantiate log
    global logger
    logger = logging.getLogger()
    init_logging(DEBUG_MODE)

    # Get config
    thales_conf = init_config(TH_FULL_CONF)
    logger.debug("Start AGI")
    try:
        if len(sys.argv) < 3:
            report_error(agi, 'wrong number of arguments')

        if sys.argv[2] == 'subscribe':
            length_prefix = len(str(thales_conf['call_pfix_aff']))
        elif sys.argv[2] == 'forward':
            length_prefix = len(str(thales_conf['call_pfix_fwd']))
        elif sys.argv[2] == 'unsubscribe':
            length_prefix = len(str(thales_conf['call_pfix_deaff']))
        else:
            report_error(agi, 'Undefined request')
        length_code = 4
        length_number_input = thales_conf['subscriber_phone_number_length'] - 1
        length_called_number = length_prefix + length_code + length_number_input

        calledNumber = sys.argv[1].strip()
        if len(calledNumber) != length_called_number and sys.argv[2] != 'forward':
            report_error(agi, f'The number of digits transmitted ({len(calledNumber)}) is not correct ({length_called_number})')
        elif len(calledNumber) == (length_called_number + length_number_input) and sys.argv[2] == 'forward':
            forwardNumber = calledNumber[length_called_number:]
        elif sys.argv[2] == 'forward':
            report_error(agi, f'The number of digits transmitted ({len(calledNumber)}) is not correct ({length_called_number + length_number_input})')

        personalCode = calledNumber[length_prefix:length_prefix+length_code]
        subscriberNumber = calledNumber[length_prefix+length_code:length_called_number]
        logger.debug(f'Personnal code : {personalCode}, subscriber number: {subscriberNumber}')
        personalCode = hashlib.sha1(personalCode.encode('utf-8')).hexdigest()

        subscriber_enum_formatted = format_subscriber_for_enum(subscriberNumber)

        thales_resolver = dns.resolver.Resolver()
        NAPTR_output = dns_lookup(agi, thales_conf, thales_resolver, 'NAPTR', subscriber_enum_formatted)
        TXT_output = dns_lookup(agi, thales_conf, thales_resolver, 'TXT', subscriber_enum_formatted)

        if TXT_output is not None:
            if not code_match(personalCode, TXT_output):
                report_error(agi, "The personal code used is not correct", -2)

        keyring = dns.tsigkeyring.from_text({f"{thales_conf['dns_key_name']}.": f"{thales_conf['dns_key_secret']}"})
        update = dns.update.UpdateMessage(zone=f"{thales_conf['dns_zone']}", keyring=keyring, keyalgorithm=f"{thales_conf['dns_key_algo']}")

        if sys.argv[2] == 'subscribe':
            update = subscribing_user(thales_conf, update, subscriberNumber, NAPTR_output, TXT_output, subscriber_enum_formatted, personalCode)
        elif sys.argv[2] == 'forward':
            update = subscribing_user(thales_conf, update, forwardNumber, NAPTR_output, TXT_output, subscriber_enum_formatted, personalCode)
        elif sys.argv[2] == 'unsubscribe':
            if NAPTR_output is None:
                report_error(agi, "The user is not subscribed", -3)
            update = unsubscribing_user(thales_conf, update, subscriberNumber, NAPTR_output, subscriber_enum_formatted)

        if update is not None:
            logger.debug(f"Executing update :\n{update}")
            dns.query.tcp(update, f"{thales_conf['dns_host']}")
        agi.set_variable('user-suscribe-result', '0')

    except Exception:
        exc_type, exc_value, exc_traceback = sys.exc_info()
        cause = repr(traceback.format_exception(exc_type, exc_value, exc_traceback))
        report_error(agi, cause)


if __name__ == '__main__':
    main()
