import json
import unittest
from hamcrest import assert_that
from hamcrest import equal_to
from mock import Mock, patch, call
import importlib

acs_request = importlib.import_module("agi.var.lib.asterisk.agi-bin.acs-request")


class TestACSRequest(unittest.TestCase):

    def setUp(self) -> None:
        self.agi = Mock()
        self.config = {}
        self.src_number =  00000
        self.dst_number = 11111
        acs_request.logger = Mock()

    def tearDown(self) -> None:
        pass

    @patch("agi.var.lib.asterisk.agi-bin.acs-request.handle_dnsformat")
    @patch("agi.var.lib.asterisk.agi-bin.acs-request.get_rights")
    def test_check_rights_unauth_401(self, mock_get_rights, mock_handle_dnsformat):
        class MockResponse(Mock):
            status_code = 401
        
        response = MockResponse()
        mock_get_rights.return_value = response

        name = 'TH_ACS_RIGHT'
        value = 'NOK'

        acs_request.handle_check_rights(self.agi, self.config, self.src_number, self.dst_number)

        self.agi.set_variable.assert_called_once_with(name, value)
        mock_handle_dnsformat.assert_not_called()

    @patch("agi.var.lib.asterisk.agi-bin.acs-request.handle_dnsformat")
    @patch("agi.var.lib.asterisk.agi-bin.acs-request.get_rights")
    @patch("agi.var.lib.asterisk.agi-bin.acs-request.logger")
    def test_check_rights_unauth_400(self, mock_logger, mock_get_rights, mock_handle_dnsformat):
        class MockResponse(Mock):
            status_code = 400

        response = MockResponse()
        mock_get_rights.return_value = response

        name = 'TH_ACS_RIGHT'
        value = 'NOK'

        acs_request.handle_check_rights(self.agi, self.config, self.src_number, self.dst_number)

        self.agi.set_variable.assert_called_once_with(name, value)
        mock_handle_dnsformat.assert_not_called()

    @patch("agi.var.lib.asterisk.agi-bin.acs-request.handle_dnsformat")
    @patch("agi.var.lib.asterisk.agi-bin.acs-request.get_rights")
    @patch("agi.var.lib.asterisk.agi-bin.acs-request.logger")
    def test_check_rights_auth(self, mock_logger, mock_get_rights, mock_handle_dnsformat):
        class MockResponse(Mock):
            status_code = 200

            def json(self):
                return {'communicationPriority': 'Routine',
                        'dnsformat': 'toto'
                        }

        response = MockResponse()
        mock_get_rights.return_value = response

        expected_calls = [
            call('TH_ACS_RIGHT', 'OK'),
            call('TH_ACS_PRIORITY', 'Routine')]

        acs_request.handle_check_rights(self.agi, self.config, self.src_number, self.dst_number)

        self.agi.set_variable.assert_has_calls(expected_calls)
        mock_handle_dnsformat.assert_called_with(self.agi ,'toto')

    @patch("agi.var.lib.asterisk.agi-bin.acs-request.check_subscriber")
    def test_subscriber_found(self, mock_check_subscriber):
        class MockResponse(Mock):
           status_code = 200

        response = MockResponse()
        mock_check_subscriber.return_value = response

        acs_request.handle_check_subscriber(self.agi, self.config, self.src_number)

        self.agi.set_variable.assert_called_with('TH_ACS_AFF_VALIDATION', 'OK')

    @patch("agi.var.lib.asterisk.agi-bin.acs-request.check_subscriber")
    def test_subscriber_banned(self, mock_check_subscriber):
        class MockResponse(Mock):
           status_code = 401

        response = MockResponse()
        mock_check_subscriber.return_value = response

        acs_request.handle_check_subscriber(self.agi, self.config, self.src_number)

        self.agi.set_variable.assert_called_with('TH_ACS_AFF_VALIDATION', 'BANNED')

    @patch("agi.var.lib.asterisk.agi-bin.acs-request.check_subscriber")
    def test_subscriber_doesnt_exist(self, mock_check_subscriber):
        class MockResponse(Mock):
           status_code = 400

        response = MockResponse()
        mock_check_subscriber.return_value = response

        acs_request.handle_check_subscriber(self.agi, self.config, self.src_number)

        self.agi.set_variable.assert_called_with('TH_ACS_AFF_VALIDATION', 'NOK')

    @patch("agi.var.lib.asterisk.agi-bin.acs-request.check_subscriber")
    def test_subscriber_error(self, mock_check_subscriber):
        class MockResponse(Mock):
           status_code = 500

        response = MockResponse()
        mock_check_subscriber.return_value = response

        acs_request.handle_check_subscriber(self.agi, self.config, self.src_number)

        self.agi.set_variable.assert_called_with('TH_ACS_AFF_VALIDATION', 'NOK')


    def test_find_non_junction(self):
        dnsformat = "0.1.3.4.8@dynamique.local"
        self.assertFalse(acs_request.detect_junction(dnsformat))
 

    def test_find_junction(self):
        dnsformat = "*.1.3.4.8@dynamique.local"
        self.assertTrue(acs_request.detect_junction(dnsformat))

    def test_dns_format_split_user_and_domain(self):
        dnsformat = "*.1.3.4.8@dynamique.local"
        expected = ["*.1.3.4.8","dynamique.local"]
        self.assertEqual(acs_request.get_user_and_domain(dnsformat),expected)
        dnsformat = "5.4.3.2.1@ailleurs.local"
        expected = ["5.4.3.2.1","ailleurs.local"]
        self.assertEqual(acs_request.get_user_and_domain(dnsformat),expected)

    def test_handle_dnsformat(self):
        dnsformat = "*.1.3.4.8@dynamique.local"

        expected_calls = [
            call('TH_ACS_IS_JCT', 'TRUE'),
            call('TH_ACS_DNSFORMAT_USER', '*.1.3.4.8'),
            call('TH_ACS_DNSFORMAT_DOMAIN','dynamique.local')
        ]
        acs_request.handle_dnsformat(self.agi, dnsformat)
        self.agi.set_variable.assert_has_calls(expected_calls)

    def test_handle_non_junction_call(self):
        dnsformat = "5.4.3.2.1@ailleurs.local"

        expected_calls = [
            call('TH_ACS_IS_JCT', 'FALSE'),
            call('TH_ACS_DNSFORMAT_USER', '5.4.3.2.1'),
            call('TH_ACS_DNSFORMAT_DOMAIN','ailleurs.local')
        ]
        acs_request.handle_dnsformat(self.agi, dnsformat)
        self.agi.set_variable.assert_has_calls(expected_calls)