#/usr/bin/python3

import unittest
from hamcrest import assert_that
from hamcrest import equal_to
import importlib
import mock

affiliation = importlib.import_module("agi.var.lib.asterisk.agi-bin.affiliation-peer-number")


class TestDBAffiliation(unittest.TestCase):

    @mock.patch("psycopg2.connect")
    def setUp(self, mock_connect):
        self.connexion = mock_connect.return_value
        self.cursor = self.connexion.cursor.return_value
        self.number = 27700911
        self.peer = 'w1kcd23'

    def tearDown(self) -> None:
        pass

    @mock.patch("agi.var.lib.asterisk.agi-bin.affiliation-peer-number.AGI")
    def test_search_peer(self, mock_agi):
        self.cursor.fetchall.return_value = [(self.peer,)]
        expected_request = "SELECT peer_sip FROM affiliation WHERE subscriber_number = %s"
        expected_dynamic_parameters = (self.number,)
        returned_item = affiliation.find_peer_by_number(mock_agi, self.cursor, self.number, affiliation.return_item)
        self.cursor.execute.assert_called_with(expected_request, expected_dynamic_parameters)
        assert_that(returned_item, equal_to(self.peer))

    @mock.patch("agi.var.lib.asterisk.agi-bin.affiliation-peer-number.AGI")
    def test_search_number(self, mock_agi):
        self.cursor.fetchall.return_value = [(self.number,)]
        expected_request = "SELECT subscriber_number FROM affiliation WHERE peer_sip = %s"
        expected_dynamic_parameters = (self.peer,)
        returned_item = affiliation.find_number_by_peer(mock_agi, self.cursor, self.peer, affiliation.return_item)
        self.cursor.execute.assert_called_with(expected_request, expected_dynamic_parameters)
        assert_that(returned_item, equal_to(self.number))

    @mock.patch("agi.var.lib.asterisk.agi-bin.affiliation-peer-number.AGI")
    def test_delete(self, mock_agi):
        expected_request = "DELETE FROM affiliation WHERE peer_sip = %s"
        expected_dynamic_parameters = (self.peer,)
        affiliation.delete(mock_agi, self.cursor, self.peer)
        self.cursor.execute.assert_called_with(expected_request, expected_dynamic_parameters)

    @mock.patch("agi.var.lib.asterisk.agi-bin.affiliation-peer-number.AGI")
    @mock.patch("agi.var.lib.asterisk.agi-bin.affiliation-peer-number.find_number_by_peer")
    @mock.patch("agi.var.lib.asterisk.agi-bin.affiliation-peer-number.find_peer_by_number")
    @mock.patch("agi.var.lib.asterisk.agi-bin.affiliation-peer-number.delete")
    def test_insert(self, mock_agi, mock_num2peer, mock_peer2num, mock_delete):
        mock_num2peer.return_value = None
        mock_peer2num.return_value = None
        expected_request = "INSERT INTO affiliation VALUES(%s,%s)"
        expected_dynamic_parameters = (self.peer,self.number)
        affiliation.insert(mock_agi, self.cursor, self.peer, self.number)
        self.cursor.execute.assert_called_with(expected_request,expected_dynamic_parameters)

    @mock.patch("agi.var.lib.asterisk.agi-bin.affiliation-peer-number.AGI")
    def test_export_item_to_agi_no_result(self, mock_agi):
        self.cursor.fetchall.return_value = []
        affiliation.export_item_to_agi(mock_agi, self.cursor)
        mock_agi.verbose.assert_called_with('item not found')
        mock_agi.set_variable.assert_called_with('TH_AFF_OP_SUCCESSFUL', 'NOK')

    @mock.patch("agi.var.lib.asterisk.agi-bin.affiliation-peer-number.AGI")
    def test_export_item_to_agi_single_number(self, mock_agi):
        self.cursor.fetchall.return_value = [(self.number,)]
        affiliation.export_item_to_agi(mock_agi, self.cursor)
        assert not mock_agi.verbose.called
        mock_agi.set_variable.assert_has_calls([mock.call('TH_AFF_OP_SUCCESSFUL', 'OK'), mock.call('TH_AFF_RETURNED_VALUE', self.number)])

    @mock.patch("agi.var.lib.asterisk.agi-bin.affiliation-peer-number.AGI")
    def test_export_item_to_agi_single_peer(self, mock_agi):
        self.cursor.fetchall.return_value = [(self.peer,)]
        affiliation.export_item_to_agi(mock_agi, self.cursor)
        assert not mock_agi.verbose.called
        mock_agi.set_variable.assert_has_calls([mock.call('TH_AFF_OP_SUCCESSFUL', 'OK'), mock.call('TH_AFF_RETURNED_VALUE', self.peer)])

    @mock.patch("agi.var.lib.asterisk.agi-bin.affiliation-peer-number.AGI")
    def test_export_item_to_agi_too_many_arguments(self, mock_agi):
        another_peer = 'labyrinth1357'
        self.cursor.fetchall.return_value = [(self.peer,),(another_peer,)]
        affiliation.export_item_to_agi(mock_agi, self.cursor)
        mock_agi.verbose.assert_called_with('unique constraint broken')
        mock_agi.set_variable.assert_called_with('TH_AFF_OP_SUCCESSFUL', 'NOK')
