from unittest import TestCase
from unittest.mock import mock_open, patch
from hamcrest import assert_that
from hamcrest import equal_to
from mock import Mock

import json
from . import thales_create_config

PLAN_NUM_STRING_ONLY = '''
{
  "dial_plan": {
    "operator_speed_dial": "9",
    "operator_number": "27761009",
    "subscriber_phone_number_length": "8",
    "call_service": "7",
    "prefixes": {
      "affiliation" : "00",
      "deaffiliation" : "01",
      "call_forward": "10"
    },
    "pnia_prefixes": {
      "zp": "77",
      "fs": "43",
      "iniz": "903803",
      "cc": "01"
    }
  },
  "local_node_id": "610"
}
'''

PLAN_NUM_INT_STR_MIXED = '''
{
  "dial_plan": {
    "operator_speed_dial": 9,
    "operator_number": 27761009,
    "subscriber_phone_number_length": 8,
    "call_service": 7,
    "prefixes": {
      "affiliation" : "00",
      "deaffiliation" : "01",
      "call_forward": 10
    },
    "pnia_prefixes": {
      "zp": "77",
      "fs": "43",
      "iniz": "903803",
      "cc": "01"
    }
  },
  "local_node_id": 610
}
'''

TSIG_KEY_FAKE_CONTENT_ONELINER = 'key "thales" {algorithm toto; secret "alice==";};'
TSIG_KEY_FAKE_CONTENT = '''
key "thales" {
algorithm toto;
secret "alice==";
};
'''
TSIG_KEY_CONTENT = '''
key "thales.local" {
    algorithm hmac-sha512;
    secret "agjZXUR3l/OmhBI9fHqVsVI8qzkt3coMUaEnLvtsCeuw4jXEhqiQ==";
};
'''

class TestThalesCreateConfig(TestCase):

    def test_load_config_netiface(self):
        class MockSession(Mock):
            address = '10.0.0.1'

        session = MockSession()
        result = thales_create_config.load_config_netiface(session)

        assert_that(result, equal_to({
            'station_ip': '10.0.0.1',
        }))

    def test_load_config_acs(self):
        def my_get(arg1, arg2):
            conf = { 'ACS':
                        { 'SERVER_IP': '127.0.0.1',
                          'SERVER_PORT': '8888'}}
            return conf[arg1][arg2]

        config = Mock()
        config.get.side_effect = my_get
        result = thales_create_config.load_config_acs(config)

        assert_that(result, equal_to({
            'acs_host': '127.0.0.1',
            'acs_port': 8888
        }))

    def test_load_config_mdt(self):
        def my_get(arg1, arg2):
            conf = { 'STATION':
                        { 'MDT_HOST': '127.0.0.1',
                          'MDT_TRANSIT_PORT': '8888',
                          'MDT_HUB_PORT' : '9999'}}
            return conf[arg1][arg2]

        config = Mock()
        config.get.side_effect = my_get
        result = thales_create_config.load_config_mdt(config)

        assert_that(result, equal_to({
            'mdt_host': '127.0.0.1',
            'mdt_port': 8888,
            'mdt_hub_port': 9999
        }))

    def test_load_config_dns(self):
        def my_get(arg1, arg2, fallback=60):
            conf = { 'STATION':
                        { 'DNS_HOST': '172.20.0.2',
                          'DNS_PORT': '53',
                          'DNS_ZONE': 'myzone',
                          'DNS_TTL': '60',
                          'DNS_KEYFILE': '/var/local/thales/dns/keys.conf'
                        }}
            return conf[arg1][arg2]

        config = Mock()
        config.get.side_effect = my_get
        result = thales_create_config.load_config_dns(config)

        assert_that(result, equal_to({
            'dns_host': '172.20.0.2',
            'dns_port': 53,
            'dns_zone': 'myzone',
            'dns_ttl': 60,
            'dns_keyfile': '/var/local/thales/dns/keys.conf'
        }))

    def test_load_config_dialplan_with_str_in_values(self):
        plan_num = json.loads(PLAN_NUM_STRING_ONLY)
        result = thales_create_config.load_config_plannum(plan_num)

        assert_that(result, equal_to({
            'operator_speed_dial': 9,
            'operator_number': 27761009,
            'subscriber_phone_number_length': 8,
            'call_pfix_aff': 700,
            'call_pfix_deaff': 701,
            'call_pfix_fwd': 710,
            'pnia_pfix_zp': '77',
            'pnia_pfix_fs': '43',
            'pnia_pfix_iniz': '903803',
            'pnia_pfix_cc': '01',
            'local_node_id': 610
        }))

    def test_load_config_dialplan_with_mixed_int_str_in_values(self):
        plan_num = json.loads(PLAN_NUM_INT_STR_MIXED)
        result = thales_create_config.load_config_plannum(plan_num)

        assert_that(result, equal_to({
            'operator_speed_dial': 9,
            'operator_number': 27761009,
            'subscriber_phone_number_length': 8,
            'call_pfix_aff': 700,
            'call_pfix_deaff': 701,
            'call_pfix_fwd': 710,
            'pnia_pfix_zp': '77',
            'pnia_pfix_fs': '43',
            'pnia_pfix_iniz': '903803',
            'pnia_pfix_cc': '01',
            'local_node_id': 610
        }))

    def test_parse_dns_tsig_key(self):
        with patch('builtins.open', mock_open(read_data=TSIG_KEY_FAKE_CONTENT_ONELINER)) as m:
            result = thales_create_config.load_dns_tsig_key_info(m)

        expected = {
          'dns_key_name' : 'thales',
          'dns_key_algo': 'toto',
          'dns_key_secret' : 'alice=='
        }
        assert_that(result, equal_to(expected))

    def test_parse_dns_tsig_multi_line_key(self):
        with patch('builtins.open', mock_open(read_data=TSIG_KEY_FAKE_CONTENT)) as m:
            result = thales_create_config.load_dns_tsig_key_info(m)

        expected = {
          'dns_key_name' : 'thales',
          'dns_key_algo': 'toto',
          'dns_key_secret' : 'alice=='
        }
        assert_that(result, equal_to(expected))

    def test_parse_dns_tsig_real_keyfile(self):
        with patch('builtins.open', mock_open(read_data=TSIG_KEY_CONTENT)) as m:
            result = thales_create_config.load_dns_tsig_key_info(m)

        expected = {
          'dns_key_name' : 'thales.local',
          'dns_key_algo': 'hmac-sha512',
          'dns_key_secret' : 'agjZXUR3l/OmhBI9fHqVsVI8qzkt3coMUaEnLvtsCeuw4jXEhqiQ=='
        }
        assert_that(result, equal_to(expected))