#!/usr/bin/python3

import argparse
import configparser
import json

from jinja2 import Environment, FileSystemLoader
from xivo_dao import init_db_from_config, default_config
from xivo_dao.alchemy.netiface import Netiface
from xivo_dao.alchemy.staticsip import StaticSIP
from xivo_dao.helpers.db_utils import session_scope

TH_DIR = '/etc/thales'
TH_CONF = f'{TH_DIR}/thales-routing.conf'
TH_PLAN_NUM = f'{TH_DIR}/plan_numerotation.json'
TH_FULL_CONF = f'{TH_DIR}/thales-conf.json'

TH_TPL_PATH = f'{TH_DIR}/templates/'
TH_TPL_CONF = [
                {
                    'tpl': 'dialplan/thales.conf',
                    'output': '/etc/asterisk/extensions_extra.d/thales.conf'},
                {
                    'tpl': 'dialplan/thales-subr.conf',
                    'output': '/etc/asterisk/extensions_extra.d/thales-subr.conf'}
            ]

# Thales config object example
"""
thales_config = 
{
    'station_ip': '10.32.0.2',
    'acs_host': '172.19.0.6',
    'acs_port': 5080,
    'mdt_host': '127.0.0.1',
    'mdt_port': 45060,
    'mdt_hub_port': 25060,
    'dns_host': '172.20.0.2',
    'dns_port': 53,
    'dns_zone': 'dynamique.local',
    'dns_ttl': 60,
    'dns_keyfile': '/etc/bind/tsig.conf',
    'dns_key_name' : 'thales.local',
    'dns_key_algo': 'hmac-sha512',
    'dns_key_secret' : 'agjZXUR3l/OmhBI9fHqVsVI8qzkt3coMUaEnLvtsCeuw4jXEhqiQ==',
    'operator_speed_dial': 9,
    'operator_number': 27761009,
    'subscriber_phone_number_length': 8,
    'call_pfix_aff': 700,
    'call_pfix_deaff': 701,
    'call_pfix_fwd': 710,
    'pnia_pfix_zp': '77',
    'pnia_pfix_fs': '43',
    'pnia_pfix_iniz': '903803',
    'pnia_pfix_cc': '33',
    'local_node_id': 610
}
"""

def change_bind_ip(ip):
    with session_scope() as session:
        row = session.query(StaticSIP).filter(StaticSIP.var_name == 'udpbindaddr').first()
        row.var_val = ip
        session.add(row)

def create_dialplan_from_template(config, tpl, output):
    with open(output, 'w') as output:
        output.write(tpl.render(thales_config=config))

def load_config(args):
    result = {}
    with session_scope() as session:
        result.update(load_config_netiface(session))

    th_config = configparser.ConfigParser()
    th_config.read(args.config)
    result.update(load_config_acs(th_config))
    result.update(load_config_mdt(th_config))
    result.update(load_config_dns(th_config))

    result.update(load_dns_tsig_key_info(result['dns_keyfile']))

    with open(args.plan_num, 'r') as configfile:
        result.update(load_config_plannum(json.load(configfile)))

    return result

def load_config_netiface(session):
    netiface = session.query(
        Netiface.address
    ).filter(
        Netiface.networktype == 'voip'
    ).first()

    return {
        'station_ip': netiface.address,
    }

def load_config_acs(config):
    return {
        'acs_host': config.get('ACS', 'SERVER_IP'),
        'acs_port': int(config.get('ACS', 'SERVER_PORT'))
    }

def load_config_mdt(config):
    return {
        'mdt_host': config.get('STATION', 'MDT_HOST'),
        'mdt_port': int(config.get('STATION', 'MDT_TRANSIT_PORT')),
        'mdt_hub_port': int(config.get('STATION', 'MDT_HUB_PORT'))
    }

def load_config_dns(config):
    return {
        'dns_host': config.get('STATION', 'DNS_HOST'),
        'dns_port': int(config.get('STATION', 'DNS_PORT')),
        'dns_zone': config.get('STATION', 'DNS_ZONE'),
        'dns_ttl': int(config.get('STATION', 'DNS_TTL', fallback=60)),
        'dns_keyfile': config.get('STATION', 'DNS_KEYFILE')
    }

def load_config_plannum(data):
    return {
        'operator_speed_dial': int(data['dial_plan']['operator_speed_dial']),
        'operator_number': int(data['dial_plan']['operator_number']),
        'subscriber_phone_number_length': int(data['dial_plan']['subscriber_phone_number_length']),
        'call_pfix_aff': int(str(data['dial_plan']['call_service'])+str(data['dial_plan']['prefixes']['affiliation'])),
        'call_pfix_deaff': int(str(data['dial_plan']['call_service'])+str(data['dial_plan']['prefixes']['deaffiliation'])),
        'call_pfix_fwd': int(str(data['dial_plan']['call_service'])+str(data['dial_plan']['prefixes']['call_forward'])),
        'pnia_pfix_zp': str(data['dial_plan']['pnia_prefixes']['zp']),
        'pnia_pfix_fs': str(data['dial_plan']['pnia_prefixes']['fs']),
        'pnia_pfix_iniz': str(data['dial_plan']['pnia_prefixes']['iniz']),
        'pnia_pfix_cc': str(data['dial_plan']['pnia_prefixes']['cc']),
        'local_node_id': int(data['local_node_id'])
    }

def clean_text(text):
    # Replace all spurious chars by space
    return (text.replace('\n', ' ') # newlines for multiline files
                .replace('{', ' ')
                .replace('}', ' ')
                .replace(';', ' ')
                .replace('"', ' ')) # also " because some val are enclosed in " and some are not

def find_value_for_key(text, key):
    # Input:
    #   key keyname algorithm algoname secret secretkey==
    # We transform it to a list:
    #   ['key', 'keyname', 'algorithm', 'algoname', 'secret', 'secretkey==']
    # and then find the key's index and take at index + 1

    # split without param so runs of consecutive whitespace are regarded as a single separator
    text_to_list = text.split()
    return text_to_list[text_to_list.index(key) + 1]

def load_dns_tsig_key_info(keyfilepath):
    result = {}
    with open(keyfilepath, 'r') as keyfile:
        text = keyfile.read()

    cleaned_text = clean_text(text)
    result.update({ 'dns_key_name' : find_value_for_key(cleaned_text, "key")})
    result.update({ 'dns_key_algo' : find_value_for_key(cleaned_text, "algorithm")})
    result.update({ 'dns_key_secret' : find_value_for_key(cleaned_text, "secret")})
    return result

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-c',
                        '--config',
                        action='store',
                        default=TH_CONF,
                        help="File where to take thales conf. Default: %(default)s")
    parser.add_argument('-p',
                        '--plan-num',
                        action='store',
                        default=TH_PLAN_NUM,
                        help="File where to take dialplan conf. Default: %(default)s")
    args = parser.parse_args()

    # Init db
    init_db_from_config(default_config())


    thales_config = load_config(args)

    # Write config
    with open(TH_FULL_CONF, "w") as output:
        json.dump(thales_config, output)

    # Create template
    env = Environment(
        loader=FileSystemLoader(TH_TPL_PATH))

    for tpl in TH_TPL_CONF:
        create_dialplan_from_template(thales_config, env.get_template(tpl['tpl']), tpl['output'])
    change_bind_ip(thales_config['station_ip'])


if __name__ == '__main__':
    main()