#!/usr/bin/python3
# -*- coding: UTF-8 -*-
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from future import standard_library
standard_library.install_aliases()
from builtins import str
from builtins import object

import sys
import requests
import logging
import traceback
import json
from xivo.agi import AGI
TH_DIR = '/etc/thales'
TH_FULL_CONF = f'{TH_DIR}/thales-conf.json'

# User configuration
DEBUG_MODE = False
LOGFILE = '/var/log/thales-acs-requests.log'
# timeout of the tcp connection to the webserver (seconds)
CONNECTION_TIMEOUT = 2

# Factory configuration
URL_HEADERS = { 'User-Agent' : 'XiVO Webservice AGI' }

logger = None
class Syslogger(object):

    def write(self, data):
        global logger
        logger.error(data)

def init_config(configpath):
    result = {}
    with open(configpath, 'r') as configfile:
        result.update(json.load(configfile))
    return result

def init_logging(debug_mode):
    if debug_mode:
        logger.setLevel(logging.DEBUG)
    else:
        logger.setLevel(logging.INFO)
    
    logfilehandler = logging.FileHandler(LOGFILE)
    formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    logfilehandler.setFormatter(formatter)
    logger.addHandler(logfilehandler)
    
    syslogger = Syslogger()
    sys.stderr = syslogger

def logAgiFile(agi, message, error = False):
    agi.verbose(message)
    if error == False:
        logger.info(message)
    else:
        logger.error(message)


def send_request(agi, url, customHeaders, params):
    result = None
    logAgiFile(agi, f'Calling ACS API: {url}')
    try:
        result = requests.get(url,
                            headers = customHeaders,
                            params = params,
                            timeout=CONNECTION_TIMEOUT)

    except requests.exceptions.ConnectionError:
        logAgiFile(agi, f"ACS Server refused connection")

    except requests.exceptions.Timeout:
        logAgiFile(agi, f"Timeout when connecting to the ACS server.")

    except:
        logAgiFile(agi, f"Unknown problem during connection to the ACS server.")
        exc_type, exc_value, exc_traceback = sys.exc_info()
        logger.debug(repr(traceback.format_exception(exc_type, exc_value,
                                                     exc_traceback)))
    finally:
        logAgiFile(agi, f'Result of ACS API call to {url}')
        if result is not None:
            logAgiFile(agi, f' - result status code: {result.status_code}')
        else:
            logAgiFile(agi, f' - no result from API call')
        return result


def get_precedence(agi, config, phonenumber):
    params = {'userSymbolicAddress': phonenumber}
    headers = {'content-type': 'application/json'}
    #headers = {'token': token}
    logAgiFile(agi, f"Precedence request for phonenumber: {phonenumber}")
    realPrecedenceURL = f"{config['acs_server_url']}{config['precedence_endpoint']}?userSymbolicAddress={phonenumber}"
    res = send_request(agi, realPrecedenceURL, headers, params)
    return res.json()

def get_rights(agi, config, src_number, dst_number):
    params = {'sourceSymbolicAddress': src_number, 'destinationSymbolicAddress': dst_number}
    headers = {'content-type': 'application/json'}
    logAgiFile(agi, f"Precedence request for src_number: {src_number} to dst_number: {dst_number}")
    res = send_request(agi, f"{config['acs_server_url']}{config['precedence_rights_endpoint']}", headers, params)
    logAgiFile(agi,res.json())
    return res.json()

def check_subscriber(agi, config, subscriber_number):
    params = {'userSymbolicAddress': subscriber_number}
    headers = {'content-type': 'application/json'}
    logAgiFile(agi, f"Check subscriber request for number: {subscriber_number}")
    res = send_request(agi, f"{config['acs_server_url']}{config['check_subscriber_number_endpoint']}", headers, params)
    logAgiFile(agi,res.json())
    return res.json()

def main():
    #init AGI and logger
    agi = AGI()
    global logger
    logger = logging.getLogger()
    init_logging(DEBUG_MODE)

    #get and complete configuration
    thales_conf = init_config(TH_FULL_CONF)

    acs_server_url = f"http://{thales_conf['acs_host']}:{thales_conf['acs_port']}"
    thales_conf['acs_server_url'] = acs_server_url
    thales_conf['auth_endpoint'] = '/api/v1/auth'    ##number= & password=
    thales_conf['precedence_endpoint'] = '/api/v1/precedenceRequest'   ###?userSymbolicAddress=20
    thales_conf['precedence_rights_endpoint'] = '/api/v1/checkCommunicationRightsRequest'     ###?sourceSymbolicAddress=23600349&destinationSymbolicAddress=23600350
    thales_conf['check_subscriber_number_endpoint'] = '/api/v1/checkUserDeclaration' ###?userSymbolicAddress=23600010

    if(len(thales_conf['acs_host']) <= 0):
        logAgiFile(agi,'ACS address is not defined')
        logAgiFile(agi,'Please configure the address in the /etc/thales/thales-routing.conf file')
        sys.exit(2)

    try:
        if len(sys.argv) < 2:
            logAgiFile(agi, f"Wrong number of arguments", error=True)
            sys.exit(1)
        if sys.argv[1] == 'getToken':
            pass
        elif sys.argv[1] == 'precedenceRequest':
            if len(sys.argv) < 3:
                logAgiFile(agi, 'Wrong number of arguments', error=True)
                sys.exit(1)
            precedence = get_precedence(agi, thales_conf, sys.argv[2])
            agi_precedence = precedence['communicationPrecedence']
            agi.set_variable('agi_call_precedence',agi_precedence)
            logAgiFile(agi, f"OK: Precedence for {sys.argv[2]} is {precedence}")
        elif sys.argv[1] == 'checkRights':
            rights = get_rights(agi, thales_conf, sys.argv[2], sys.argv[3])
            agi_precedence = rights['communicationPrecedence']
            agi_right = rights['result']
            agi.set_variable('agi_right',agi_right)
            agi.set_variable('agi_call_precedence',agi_precedence)
            logAgiFile(agi, f"GOT: Rights {agi_right} - {agi_precedence}")
        elif sys.argv[1] == 'checkSubscriber':
            exist = check_subscriber(agi, thales_conf, sys.argv[2])
            acs_validates_number = exist['result']
            agi.set_variable('acs_validates_number', acs_validates_number)
            logAgiFile(agi, f"GOT: Exist {exist}")
        else:
            logAgiFile(agi,'Undefined request', error=True)
            sys.exit(1)
    except Exception:
        exc_type, exc_value, exc_traceback = sys.exc_info()
        logger.error(repr(traceback.format_exception(exc_type, exc_value,
                                          exc_traceback)))
        sys.exit(1)

    sys.exit(1)


if __name__ == '__main__':
    main()
