import unittest
from hamcrest import assert_that
from hamcrest import equal_to
import mock
import re
import dns.resolver
import dns.tsigkeyring
import dns.update
import dns.rdataset
import dns.rdtypes.IN.NAPTR
import dns.rdtypes.ANY.TXT
import dns.zone
import dns.query
import importlib
import hashlib
user_subscription = importlib.import_module("agi.var.lib.asterisk.agi-bin.user-subscription")

TEST_CONF={
    "station_ip": "10.50.0.191",
    "acs_host": "172.19.0.6",
    "acs_port": 5080,
    "mdt_host": "127.0.0.1",
    "mdt_port": 45060,
    "mdt_hub_port": 25060,
    "dns_host": "172.20.0.2",
    "dns_port": 53,
    "dns_zone": "dynamique.local",
    "dns_ttl": 60,
    "dns_keyfile": "/var/local/thales/dns/keys.conf",
    "dns_key_name": "thales.local",
    "dns_key_algo": "hmac-sha512",
    "dns_key_secret": "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa==",
    "operator_speed_dial": 9,
    "operator_number": 27761009,
    "subscriber_phone_number_length": 8,
    "call_pfix_aff": 700,
    "call_pfix_deaff": 701,
    "call_pfix_fwd": 710,
    "pnia_pfix_zp": 77,
    "pnia_pfix_fs": 43,
    "pnia_pfix_iniz": 903803,
    "pnia_pfix_cc": 33,
    "local_node_id": 610
}


class TestUserSubscription(unittest.TestCase):

    def setUp(self) -> None:
        self.subscriber_number = '7700911'
        self.personalCode = '1234'
        self.hashed_personal_code = hashlib.sha1(self.personalCode.encode('utf-8')).hexdigest()
        self.formatted_subscriber_number = '1.1.9.0.0.7.7.2'
        self.keyring = dns.tsigkeyring.from_text({f"{TEST_CONF['dns_key_name']}.": f"{TEST_CONF['dns_key_secret']}"})
        self.update = dns.update.UpdateMessage(zone=f"{TEST_CONF['dns_zone']}", keyring=self.keyring, keyalgorithm=f"{TEST_CONF['dns_key_algo']}")

    def tearDown(self) -> None:
        pass

    @mock.patch("agi.var.lib.asterisk.agi-bin.user-subscription.logger")
    def test_format_subscriber_for_enum(self, mock_logger):
        assert_that(user_subscription.format_subscriber_for_enum(self.subscriber_number),
                    equal_to(self.formatted_subscriber_number))
        self.assertLogs(mock_logger, 'DEBUG')

    @mock.patch('agi.var.lib.asterisk.agi-bin.user-subscription.logger')
    def test_code_match(self, mock_logger):
        dns_lookup_answers = '''id 52128
id 34593
opcode QUERY
rcode NOERROR
flags QR AA RD RA
;QUESTION
1.3.9.0.0.7.7.2.dynamique.local. IN TXT
;ANSWER
1.3.9.0.0.7.7.2.dynamique.local. 60 IN TXT "CodePersonnel:7110eda4d09e062aa5e4a390b0a572ac0d2c0220"
;AUTHORITY
;ADDITIONAL
'''
        assert_that(user_subscription.code_match(self.hashed_personal_code, dns_lookup_answers),
                    equal_to(True))
        self.assertLogs(mock_logger, 'DEBUG')

    @mock.patch('agi.var.lib.asterisk.agi-bin.user-subscription.logger')
    def test_code_doesnt_match(self, mock_logger):
        dns_lookup_answers = '''id 52128
id 34593
opcode QUERY
rcode NOERROR
flags QR AA RD RA
;QUESTION
1.3.9.0.0.7.7.2.dynamique.local. IN TXT
;ANSWER
1.3.9.0.0.7.7.2.dynamique.local. 60 IN TXT "1234"
;AUTHORITY
;ADDITIONAL'''
        assert_that(user_subscription.code_match('7110eda4d09e062aa5e4a390b0a572ac0d2c0220', dns_lookup_answers),
                    equal_to(False))
        self.assertLogs(mock_logger, 'DEBUG')

    @mock.patch('agi.var.lib.asterisk.agi-bin.user-subscription.logger')
    def test_subscribe_user_with_txt_line(self, mock_logger):
        expected_cmd_without_id = '''opcode UPDATE
rcode NOERROR
flags 
;ZONE
dynamique.local. IN SOA
;PREREQ
;UPDATE
1.1.9.0.0.7.7.2.dynamique.local. 60 IN NAPTR 100 10 "U" "E2U+sip" "!^.*$!sip:27700911@10.50.0.191!" .
;ADDITIONAL'''
        cmd = user_subscription.subscribing_user(TEST_CONF, self.update, self.subscriber_number, None, 'exists', self.formatted_subscriber_number, self.hashed_personal_code)
        cmd = re.sub(r'id.*\n', '', cmd.to_text())
        assert_that(cmd,
                    equal_to(expected_cmd_without_id))
        self.assertLogs(mock_logger, 'DEBUG')

    @mock.patch('agi.var.lib.asterisk.agi-bin.user-subscription.logger')
    def test_subscribe_user_without_txt_line(self, mock_logger):
        expected_cmd_without_id = '''opcode UPDATE
rcode NOERROR
flags 
;ZONE
dynamique.local. IN SOA
;PREREQ
;UPDATE
1.1.9.0.0.7.7.2.dynamique.local. 60 IN NAPTR 100 10 "U" "E2U+sip" "!^.*$!sip:27700911@10.50.0.191!" .
1.1.9.0.0.7.7.2.dynamique.local. 60 IN TXT "CodePersonnel:7110eda4d09e062aa5e4a390b0a572ac0d2c0220"
;ADDITIONAL'''
        cmd = user_subscription.subscribing_user(TEST_CONF, self.update, self.subscriber_number, None, None,
                                                 self.formatted_subscriber_number, self.hashed_personal_code)
        cmd = re.sub(r'id.*\n', '', cmd.to_text())
        assert_that(cmd,
                    equal_to(expected_cmd_without_id))
        self.assertLogs(mock_logger, 'DEBUG')

    @mock.patch('agi.var.lib.asterisk.agi-bin.user-subscription.logger')
    def test_unsubscribe_user(self, mock_logger):
        expected_cmd_without_id = '''opcode UPDATE
rcode NOERROR
flags 
;ZONE
dynamique.local. IN SOA
;PREREQ
;UPDATE
1.1.9.0.0.7.7.2.dynamique.local. ANY NAPTR
;ADDITIONAL'''
        cmd = user_subscription.unsubscribing_user(TEST_CONF, self.update, self.subscriber_number, 'exists',
                                                   self.formatted_subscriber_number)
        cmd = re.sub(r'id.*\n', '', cmd.to_text())
        assert_that(cmd,
                    equal_to(expected_cmd_without_id))
        self.assertLogs(mock_logger, 'DEBUG')

    @mock.patch('agi.var.lib.asterisk.agi-bin.user-subscription.logger')
    def test_update_user(self, mock_logger):
        expected_cmd_without_id = '''opcode UPDATE
rcode NOERROR
flags 
;ZONE
dynamique.local. IN SOA
;PREREQ
;UPDATE
1.1.9.0.0.7.7.2.dynamique.local. ANY NAPTR
1.1.9.0.0.7.7.2.dynamique.local. 60 IN NAPTR 100 10 "U" "E2U+sip" "!^.*$!sip:27700911@10.50.0.191!" .
;ADDITIONAL'''
        cmd = user_subscription.subscribing_user(TEST_CONF, self.update, self.subscriber_number, 'exists', 'exists',
                                                 self.formatted_subscriber_number, self.hashed_personal_code)
        cmd = re.sub(r'id.*\n', '', cmd.to_text())
        assert_that(cmd,
                    equal_to(expected_cmd_without_id))
        self.assertLogs(mock_logger, 'DEBUG')

    @mock.patch('agi.var.lib.asterisk.agi-bin.user-subscription.logger')
    def test_forward_user(self, mock_logger):
        forward_number = '7700941'
        expected_cmd_without_id = '''opcode UPDATE
rcode NOERROR
flags 
;ZONE
dynamique.local. IN SOA
;PREREQ
;UPDATE
1.1.9.0.0.7.7.2.dynamique.local. 60 IN NAPTR 100 10 "U" "E2U+sip" "!^.*$!sip:27700941@10.50.0.191!" .
;ADDITIONAL'''
        cmd = user_subscription.subscribing_user(TEST_CONF, self.update, forward_number, None, 'exists',
                                                 self.formatted_subscriber_number, self.hashed_personal_code)
        cmd = re.sub(r'id.*\n', '', cmd.to_text())
        assert_that(cmd,
                    equal_to(expected_cmd_without_id))
        self.assertLogs(mock_logger, 'DEBUG')
